package com.releaseshub.gradle.plugin.task

object PullRequestGenerator {

    private const val PULL_REQUEST_FOOTER = "This pull request was automatically generated by **[Releases Hub Gradle Plugin](https://github.com/releaseshub/releases-hub-gradle-plugin)**"

    fun createBody(upgradeResults: List<UpgradeResult>): String {
        val builder = StringBuilder()
        builder.appendln("## Dependencies upgrades")
        addCommonText(builder, upgradeResults)
        builder.appendln()
        builder.append(PULL_REQUEST_FOOTER)
        return builder.toString()
    }

    private fun addCommonText(builder: StringBuilder, upgradeResults: List<UpgradeResult>) {
        upgradeResults.forEach {
            builder.appendln("#### ${it.artifact} `${it.artifact?.fromVersion}` -> `${it.artifact?.toVersion}`")
            var atLeastOneItem = false
            if (it.artifact?.releaseNotesUrl != null) {
                builder.append("* [Releases notes](${it.artifact.releaseNotesUrl})")
                atLeastOneItem = true
            }
            if (it.artifact?.sourceCodeUrl != null) {
                builder.append(if (atLeastOneItem) " | " else "* ")
                builder.append("[Source code](${it.artifact.sourceCodeUrl})")
                atLeastOneItem = true
            }
            if (it.artifact?.documentationUrl != null) {
                builder.append(if (atLeastOneItem) " | " else "* ")
                builder.append("[Documentation](${it.artifact.documentationUrl})")
                atLeastOneItem = true
            }
            if (atLeastOneItem) {
                builder.appendln()
            }
        }
    }

    fun createComment(upgradeResults: List<UpgradeResult>): String {
        val builder = StringBuilder()
        addCommonText(builder, upgradeResults)
        return builder.toString()
    }
}